-- This file is part of CyrodiilTickTimer
--
-- (C) 2018 FullHeartContainer
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.

CyrodiilTickTimer = CyrodiilTickTimer or {}
CyrodiilTickTimer.addonVars = {}
CyrodiilTickTimer.addonVars.version	= "4.1"
CyrodiilTickTimer.addonVars.name 		= "CyrodiilTickTimer"
CyrodiilTickTimer.addonVars.author 	= "FullHeartContainer"
CyrodiilTickTimer.addonVars.website	= "https://www.esoui.com/downloads/info2172-CyrodiilTickTimer.html"

----------------------------------------------
-- Utility
----------------------------------------------

local function bl(val)
	if val == nil then return "NIL" elseif val then return "T" else return "F" end
end

local function nn(val)
    if val == nil then return "NIL" end
    return val
end

function CyrodiilTickTimer.formatTime(delta, inclueSec, doAdditional)
    local sec = delta % 60
    delta = (delta - sec) / 60
    local min = delta % 60
    local out = min .. "m"

	if doAdditional == true then
		if out == '0m' then
			out = '<1m'
		else
			out = '  '..out
		end
	end

    if inclueSec then
        out = out .. " " .. sec .. "s"
    end

    return out
end

CyrodiilTickTimer.errors = {}

function CyrodiilTickTimer:error(val)
    if not self.errors[val] then
        self.errors[val] = 1
        d("|cFF0000ERROR (CyrodiilTickTimer): " .. val .. "\n|CCCCCCCPlease file this bug info with a screenshot at |CEEEEFFesoui.com (CyrodiilTickTimer)")
    end
end

----------------------------------------------
-- Events
----------------------------------------------



function CyrodiilTickTimer.saveWindowPosition( window )
    local _, sP, _, aP, x, y = window:GetAnchor()
    CyrodiilTickTimer.cfg.anchPoint = aP
    CyrodiilTickTimer.cfg.selfPoint = sP
    CyrodiilTickTimer.cfg.xoff = x
    CyrodiilTickTimer.cfg.yoff = y
end

function CyrodiilTickTimer.actionLayerChange(_, _, activeLayerIndex)
    CyrodiilTickTimer_UI:SetHidden(activeLayerIndex > 2)
end

----------------------------------------------
-- Notification UI pool
----------------------------------------------

CyrodiilTickTimer.entryCount = 0
CyrodiilTickTimer.entries = {}

function CyrodiilTickTimer:reconfigureLabels()
    for _,entry in pairs(self.entries) do
        --Forces reconfigure on next update
        entry.type = nil
    end
end

function CyrodiilTickTimer:hideRow(index)
    if self.entries[index] then
        self.entries[index].main:SetHidden(true)
    end
end

function CyrodiilTickTimer:getUIRow(index)
    if #self.entries < index then
        table.insert(self.entries, self.Label())
        index = #self.entries
    end

    self.entries[index].main:SetHidden(false)

    return self.entries[index]
end

function CyrodiilTickTimer:printAll()
    local i = 1

    for _,status in ipairs(self.statusBars) do
        self:getUIRow(i):update(status)
        i = i + 1
    end

    for _,battle in pairs(self.battles) do
        self:getUIRow(i):update(battle)
        i = i + 1
    end

    --Fix since auto-resize doesn't seem to work well
    self.ui:SetHeight(math.max(i*35,70))

    for j=i,#self.entries do
        self:hideRow(j)
    end
end

----------------------------------------------
-- Battle management
----------------------------------------------

CyrodiilTickTimer.battles = {}

function CyrodiilTickTimer:add(keepID)
    if self.battles[keepID] == nil then
        self.battles[keepID] = self.Battle(keepID)
    else
        self.battles[keepID]:restart()
    end
end

function CyrodiilTickTimer:checkAdd(keepID)
    if self.battles[keepID] == nil then
        local battle = self.Battle(keepID)

        if battle:isBattle() then
            self.battles[keepID] = battle
        end
    elseif self.battles[keepID]:isBattle() then
        self.battles[keepID]:restart()
    end
end

function CyrodiilTickTimer:updateAll()
    for i,_ in pairs(self.battles) do
        --Update in-place
        self.battles[i]:update()
    end

    for _,status in pairs(self.statusBars) do
        status:update()
    end
end

------------------------------------------------------------------------
-- Initialization
------------------------------------------------------------------------

CyrodiilTickTimer.visible = false

function CyrodiilTickTimer:init()
    --Init UI
    self.ui:SetHidden(false)

    --Populate data
    self:refresh()

    --Add events

    EVENT_MANAGER:RegisterForUpdate(CyrodiilTickTimer.addonVars.name .. "UIUpdate", 1000, function()
        self:printAll()
    end)

    self.visible = true

    EVENT_MANAGER:RegisterForEvent(CyrodiilTickTimer.addonVars.name, EVENT_ACTION_LAYER_POPPED, self.actionLayerChange)
    EVENT_MANAGER:RegisterForEvent(CyrodiilTickTimer.addonVars.name, EVENT_ACTION_LAYER_PUSHED, self.actionLayerChange)
end

function CyrodiilTickTimer:refresh()
    --Get initial scan
    self.battles = {}
    self.battleContext = BGQUERY_LOCAL
    self.campaign = GetCurrentCampaignId()

    --Could separate this with a data refresh eventually, but just do a hard reset for now
    self.statusBars = {}
    table.insert(self.statusBars, self.ScoringBar())

    --Force update on status bar
    self:reconfigureLabels()
end


function CyrodiilTickTimer.toggle()
    local self = CyrodiilTickTimer

    if self.visible then
        self:deinit()
    else
        self:init()
    end
end

--TODO: Properly setup on Addon init or playerLoad
--TODO: only show while in Cyrodiil?
SLASH_COMMANDS["/cyrodiilticktimer"] = CyrodiilTickTimer.toggle

--Called once. Handles controls, etc.
function CyrodiilTickTimer.addonInit()
    local self = CyrodiilTickTimer

    --Init saved variables
    local def = {
        xoff = -10,
        yoff = 60,
        trackerDisable = false,
        showPopBars = false,
    }

    self.cfg = ZO_SavedVars:NewAccountWide("CyrodiilTickTimer_SavedVars", 1.0, "config", def)

    --Create UI
    self.ui = WINDOW_MANAGER:CreateTopLevelWindow("CyrodiilTickTimer_UI")
    self.ui:SetWidth(CyrodiilTickTimer.width)
    self.ui:SetMouseEnabled(true)
    self.ui:SetMovable(true)
    self.ui:SetClampedToScreen(true)
    self.ui:SetHandler("OnMoveStop", self.saveWindowPosition)

    local _, pt, relTo, relPt = CyrodiilTickTimer_UI:GetAnchor()
    self.ui:ClearAnchors()
    self.ui:SetAnchor(CyrodiilTickTimer.cfg.selfPoint or TOPLEFT,
        GuiRoot, CyrodiilTickTimer.cfg.anchPoint or TOPRIGHT,
        CyrodiilTickTimer.cfg.xoff, CyrodiilTickTimer.cfg.yoff)

    --Create settings menu
    local LAM = LibStub("LibAddonMenu-2.0")
    LAM:RegisterAddonPanel(CyrodiilTickTimer.addonVars.name .. "-LAM", self.menuPanel)
    LAM:RegisterOptionControls(CyrodiilTickTimer.addonVars.name .. "-LAM", self.menuOptions)
    self.initLAM = true

    --[[if (GetDate() % 1000)== 401 then
        --NOTE: If you see this before 4/1, please don't share
        table.insert(self.menuOptions,{
            type = "checkbox",
            name = GetString(SI_CyrodiilTickTimer_APRIL1),
            tooltip = GetString(SI_CyrodiilTickTimer_APRIL1_TOOLTIP),
            getFunc = function() return CyrodiilTickTimer.cfg.aprOff or false end,
            setFunc = function(v) CyrodiilTickTimer.cfg.aprOff = v; CyrodiilTickTimer:refresh() end,
        })
    end]]
end

function CyrodiilTickTimer.playerInit()
    local self = CyrodiilTickTimer

    if not self.initLAM then
        self.addonInit()
    end

    if IsPlayerInAvAWorld() then
        if self.visible then
            self:refresh()
        else
            self:init()
        end
    elseif self.visible then
        self:deinit()
    end
end

EVENT_MANAGER:RegisterForEvent(CyrodiilTickTimer.addonVars.name .. "-init", EVENT_PLAYER_ACTIVATED, CyrodiilTickTimer.playerInit)
